#include <memory.h>

#include "bitmap.hh"

Bitmap::Bitmap(int w, int h)
    : width(w)
    , height(h)
    , padding(8)
{
    byteswide = ((w - 1) / padding + 1) * (padding / 8);
    data = new unsigned char [h * byteswide];
    memset(data, 0, h * byteswide);
}

Bitmap::Bitmap(unsigned char *d, int w, int h, int pad)
    : width(w)
    , height(h)
    , padding(pad)
{
    byteswide = ((w - 1) / padding + 1) * (padding / 8);
    data = new unsigned char [h * byteswide];
    memcpy(data, d, h * byteswide);
}

int
Bitmap::get_pixel(int x, int y)
{
    int byte = y * byteswide + x / 8;
    int shift = (x & 7);
    return (((data[byte] << shift) & 0x80) == 0x80);
}

void
Bitmap::set_pixel(int x, int y, int on)
{
    int byte = y * byteswide + x / 8;
    int shift = (x & 7);
    if (on)
	data[byte] |= (0x80u >> shift);
    else
	data[byte] &= (0xffffu >> shift);
}

    
