#include <iostream.h>
#include <math.h>
#include "bitmap.hh"

struct mgr_font_header
{
    unsigned char type;		/* font type (magic number) '^v' */
    unsigned char wide;		/* character width (pixels) */
    unsigned char high;		/* char height (pixels) */
    unsigned char baseline;	/* pixels from bottom of glyph */
    unsigned char count;	/* number of chars in font */
    char start;			/* starting char in font */
};

#define FONT_A '\030'		/* Fixed width, 32-bit aligned */
#define FONT_X '\026'		/* Fixed width, 16-bit aligned */
#define FONT_S '\027'		/* Proportional font (not handled) */

typedef unsigned char byte;

int
main(int argc, char *argv[])
{
    struct mgr_font_header hdr;
    istream &input = cin;
    ostream &output = cout;

    char *foundry = "Bellcore";
    char *family = 0;
    char *weight = "Medium";
    char *slant = "R";
    char *setwidth = "Normal";
    char *addstyle = "";

    int optchar;
    extern char * optarg;

    while ((optchar = getopt(argc, argv, "F:f:w:s:W:a:")) != -1)
    {
	switch (optchar)
	{
	  default:
	    cout << "usage: " << argv[0] << "[-F foundry] -f family -w weight -s slant -W setwidth -a addstyle" << endl;
	    exit(1);
	  case 'F':
	    foundry = optarg;
	    break;
	  case 'f':
	    family = optarg;
	    break;
	  case 'w':
	    weight = optarg;
	    break;
	  case 's':
	    slant = optarg;
	    break;
	  case 'W':
	    setwidth = optarg;
	    break;
	  case 'a':
	    addstyle = optarg;
	    break;
	}
    }

    if (!family)
    {
	cerr << "Must give family name" << endl;
	exit(1);
    }
    
    input.read(&hdr, sizeof(hdr));
    if (hdr.type != FONT_A && hdr.type != FONT_X)
    {
	cerr << "mgrtobdf: unknown font format\n" << endl;
	exit(1);
    }

    int pad = ((hdr.type == FONT_A) ? 32 : 16);
    int byteswide = (((hdr.wide * hdr.count) - 1) / pad + 1) * (pad / 8);
    int charbytes = (hdr.wide - 1) / 8 + 1;
    unsigned char *bmdata = new unsigned char[hdr.high * byteswide];
    input.read(bmdata, hdr.high * byteswide);
    Bitmap mgrfont(bmdata, hdr.wide * hdr.count, hdr.high, pad);

    output << "STARTFONT 2.1" << endl;
    output << "COMMENT Converted from an MGR font" << endl;
    output << "FONT -" << foundry << "-" << family << "-" << weight
	<< "-" << slant << "-" << setwidth << "-" << addstyle
	<< "-" << (int)hdr.high << "-" << 10 * (int)hdr.high
	<< "-72-72-c-60-iso8859-1" << endl;
    output << "SIZE 120 72 72" << endl;
    output << "FONTBOUNDINGBOX " << (int)hdr.wide << " " << (int)hdr.high
	   << " " << 0 << " " << (int)-hdr.baseline << endl;

    output << "STARTPROPERTIES 19" << endl;
    output << "FONTNAME_REGISTRY \"\"" << endl;
    output << "FOUNDRY \"" << foundry << "\"" << endl;
    output << "FAMILY_NAME \"" << family << "\"" << endl;
    output << "WEIGHT_NAME \"" << weight << "\"" << endl;
    output << "SLANT \"" << slant << "\"" << endl;
    output << "SETWIDTH_NAME \"" << setwidth << "\"" << endl;
    output << "ADD_STYLE_NAME \"" << addstyle << "\"" << endl;
    output << "PIXEL_SIZE " << (int)hdr.high << endl;
    output << "POINT_SIZE " << 10 * (int)hdr.high << endl;
    output << "RESOLUTION_X 72" << endl;
    output << "RESOLUTION_Y 72" << endl;
    output << "SPACING \"C\"" << endl;
    output << "AVERAGE_WIDTH 60" << endl;
    output << "CHARSET_REGISTRY \"iso8859\"" << endl;
    output << "CHARSET_ENCODING \"1\"" << endl;
    output << "DEFAULT_CHAR 0" << endl;
    output << "FONT_DESCENT " << (int)hdr.baseline << endl;
    output << "FONT_ASCENT " << (int)(hdr.high - hdr.baseline) << endl;
    output << "COPYRIGHT \"Bellcore\"" << endl;
    output << "ENDPROPERTIES" << endl;

    output << "CHARS " << (int)hdr.count << endl;
    
    for (int ch = 0; ch < hdr.count; ch++)
    {
	output << "STARTCHAR ch" << (int)(ch + hdr.start) << endl;
	output << "ENCODING " << (int)(ch + hdr.start) << endl;
	output << "SWIDTH "
	       << (int)floor(1000 * hdr.wide / hdr.high) << " 0" << endl;
	output << "DWIDTH " << (int)hdr.wide << " " << 0 << endl;
	output << "BBX " << (int)hdr.wide << " " << (int)hdr.high
	       << " " << 0 << " " << (int)-hdr.baseline << endl;
	output << "BITMAP" << endl;

	for (int y = 0; y < hdr.high; y++)
	{
	    unsigned long strip = 0;
	    for (int x = 0; x < hdr.wide; x++)
	    {
		if (mgrfont.get_pixel(ch * hdr.wide + x, y))
		{
		    strip |= (0x80000000ul >> x);
		}
	    }
	    for (int nibble = 0; nibble < charbytes * 2; nibble++)
	    {
		output << ((strip >> 28) < 10 ? (char)('0' + (strip >> 28))
			   : (char)('A' + (strip >> 28) - 10));
		strip = (strip << 4);
	    }
	    output << endl;
	}
	output << "ENDCHAR" << endl;
    }
    output << "ENDFONT" << endl;
}

    
